#
# Copyright (c) 2018 LAAS/CNRS
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software for any purpose
# with or without   fee is hereby granted, provided   that the above  copyright
# notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS  SOFTWARE INCLUDING ALL  IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS. IN NO EVENT SHALL THE AUTHOR  BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR  ANY DAMAGES WHATSOEVER RESULTING  FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION,   ARISING OUT OF OR IN    CONNECTION WITH THE USE   OR
# PERFORMANCE OF THIS SOFTWARE.
#
#                                            Anthony Mallet on Mon Sep 24 2018
#


# --- db2mail --------------------------------------------------------------
#
proc db2mail { path db tag email sendmail } {
  global tcldir

  if {$email eq ""} return
  
  set db [opendb $db]

  # generate statuses
  sqlpkgstatus $db $tag broken msg
  sqlpkgstatus $db $tag warning msg
  sqlpkgstatus $db $tag sysdep msg
  sqlpkgstatus $db $tag fixed msg
  set date [$db eval "select date from session where tag = '$tag'"]

  # send message
  set count 0
  set text [list]
  if {[llength $msg(broken)]} {
    lappend text ""
    lappend text " * Broken packages:"
    lappend text {*}$msg(broken)
    incr count [llength $msg(broken)]
  }
  if {[llength $msg(warning)]} {
    lappend text ""
    lappend text " * Packaging warnings:"
    lappend text {*}$msg(warning)
    incr count [llength $msg(warning)]
  }
  if {[llength $msg(sysdep)]} {
    lappend text ""
    lappend text " * Missing system dependencies:"
    lappend text {*}$msg(sysdep)
    incr count [llength $msg(sysdep)]
  }
  if {[llength $msg(fixed)]} {
    lappend text ""
    lappend text " * Fixed packages:"
    lappend text {*}$msg(fixed)
    incr count [llength $msg(fixed)]
  }

  if {$count} {
    set out [open "|$sendmail" w]
    puts $out [join [subst {
      "From: Robert Bulk <robotpkg@laas.fr>"
      "To: $email"
      "Subject: \[rbulk/$tag\] Automated $path status report"
      "Auto-Submitted: auto-generated"
      ""
      "Automated $path bulk build status report for $tag."
      "Report on [clock format $date], changes since last run are shown."
    }] "\n"]
    puts $out [join $text "\n"]
    close $out
    puts "Sent $count status updates to $email."
  } else {
    puts "No status update since last run."
  }

  # clear fixed status
  $db eval [subst {
    update meta set fixed = null where fixed is not null and
    id in (select id from pkg where tag = '$tag')
  }]

  # no need to analyze/vaccum
  closedb $db off
}


# --- sqlpkgstatus ---------------------------------------------------------
#
proc sqlpkgstatus { db tag status msg& } {
  upvar ${msg&} msg
  set limit 36

  set extra ""
  if {$status in {broken warning na fixed}} {
    append extra " and (sysdep is null)"
  }
  if {$status in {warning na fixed}} { append extra " and (broken is null)" }
  if {$status in {na fixed}} { append extra " and (warning is null)" }
  if {$status in {fixed}} { append extra " and (na is null)" }

  set msg($status) ""
  $db eval [subst {
    select path, pkgname,
    group_concat(distinct (
      select by.pkgname from pkg as by natural join breaks as bby
      where bby.breaks = pkg.id)) as blist,
    count(breaks) as breaks, count(broken) as broken, count(warning) as warning
    from pkg natural join meta natural left outer join breaks
    where (tag = '$tag') and
    (fixed = date_start) and ($status is not null) $extra
    group by pkgname
    order by breaks desc, broken desc, warning desc, pkgname
    limit $limit
  }] pkg {
    set breaks ""
    if {$pkg(breaks) > 0} {
      append breaks " breaks $pkg(breaks) other package"
      if {$pkg(breaks) > 1} {
        append breaks "s"
      }
    } elseif {[string length $pkg(blist)]} {
      append breaks " broken by [join [split $pkg(blist) ,] {, }]"
    }

    lappend msg($status) "   - $pkg(pkgname) ($pkg(path))$breaks"
  }

  $db eval [subst {
    select count(distinct pkgname) as n
    from pkg natural join meta
    where (fixed is not null) and ($status is not null) $extra
  }] {
    if {$n > $limit} {
      lappend msg($status) "   - [expr {$n-$limit}] more packages not shown"
    }
  }
}
package provide rbulk-report 20180927
